/*
 * Copyright (C) 2017-2018  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */
#ifndef STORAGE_HH_F547FDFC7F6B4D918928C606B9E4A87E//date "+%s" | md5sum | cut -f1 -d" " | tr "[a-f]" "[A-F]" | tr -d "\n"
#define STORAGE_HH_F547FDFC7F6B4D918928C606B9E4A87E

#include "src/i_storage.hh"
#include "src/scan_type.hh"

#include "3rd_party/sqlite3pp/headeronly_src/sqlite3pp.h"

namespace Fred {
namespace Akm {
namespace Sqlite {


class SqliteStorage : public IStorage
{
public:
    SqliteStorage(const std::string& _filename);

    void append_to_scan_queue(const DomainScanTaskCollection& _data) const;

    void append_to_scan_queue_if_not_exists(const DomainScanTaskCollection& _data) const;

    void wipe_scan_queue() const;

    void prune_scan_queue() const;

    DomainScanTaskCollection get_scan_queue_tasks() const;

    void save_scan_results(const ScanResults& _results, const DomainScanTaskCollection& _tasks, long long _iteration_id) const;

    long long start_new_scan_iteration() const;

    void end_scan_iteration(const long long _iteration_id) const;

    void wipe_unfinished_scan_iterations() const;

    long long prune_finished_scan_queue() const;

    ScanResultRows get_scan_result_rows_of_akm_insecure_candidates_for_akm_notify(
            int _seconds_back,
            bool _notify_from_last_iteration_only,
            bool _align_to_start_of_day = false) const;

    ScanResultRows get_scan_result_rows_of_akm_insecure_candidates_for_akm_turn_on(
            int _seconds_back,
            bool _align_to_start_of_day = false) const;

    ScanResultRows get_scan_result_rows_of_akm_secure_candidates_for_akm_turn_on(
            int _seconds_back,
            bool _align_to_start_of_day = false) const;

    ScanResultRows get_scan_result_rows_of_akm_members_for_update(
            int _seconds_back,
            bool _align_to_start_of_day = false) const;

    void set_domain_notified_status(const DomainNotifiedStatus& _domain_notified_status) const;

    boost::optional<DomainNotifiedStatus> get_domain_last_notified_status(
            long long _domain_id) const;

    void clean_scan_results(
            int _keep_seconds_back,
            bool _align_to_start_of_day = false) const;

    unsigned int get_current_unix_time() const;

private:
    sqlite3pp::database get_db() const;

    std::string filename_;
};


} // namespace Sqlite
} // namespace Akm
} // namespace Fred

#endif//STORAGE_HH_F547FDFC7F6B4D918928C606B9E4A87E
